/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3910_PWM
 * IDE tooling  : AndeSight C/C++ IDE, version: 3.2.1
 * Device Ver.  :
 * Library Ver. : 0.3
 * MCU Device   :
 * Description  :
 * Created Date : 2021/10/28
 * Created by   :
 *
 * Program Description:
 *                                                                            
 *  HY16F3910                 PWM                        
 *----------------          ----------------                                                                                                                                                                                              
 *                |        |                                             
 *          PT2.0 | -----> | PWM0      
 *          PT2.1 | -----> | PWM1                                         
 *                |        |                                                                                
 *----------------           ----------------                                                                                       
 * 
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvCLOCK.h"
#include "DrvLCD.h"
#include "Display.h"
#include "main.h"
#include "System.h"
#include "DrvTimer.h"
#include "DrvGPIO.h"
#include "DrvRTC.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMB2done:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_WDTdone:1;
    unsigned REV:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define HAO_4MHZ
//#define HAO_32MHZ

#define TMB1DOD_PWM1 0x7f
#define TMB1DOD_PWM0 0x3f
#define TMB2DOD_PWM3 0x7f
#define TMB2DOD_PWM2 0x3f
#define TMB1COD 0xff
#define TMB2COD 0xff
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned int sec,min,hour,week,day,month,year ;
unsigned int RTC_counter;
unsigned int TimerDtata;
MCUSTATUS  MCUSTATUSbits;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void DrvPWM_Out1(void);
void DrvPWM_Out2(void);
void DrvPWM_Out3(void);
void DrvPWM_Out4(void);
void Delay(unsigned int num);
void InitalRTC(void);
int ComputeWeek(int TempYear, int TempMonth, int TempDay);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
#if defined(HAO_4MHZ)
  DrvCLOCK_SelectIHOSC(E_HAO_4M);              //Select internal 4.147MHz
  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);   //ENMCD=11b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
  DrvCLOCK_CalibrateHAO(TRIM_HAO4MHZ);         //Calibration 4.147MHz
#elif defined(HAO_32MHZ)
  DrvCLOCK_SelectIHOSC(E_HAO_32M);             //Select internal 31.795MHz
  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);   //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
  DrvCLOCK_CalibrateHAO(TRIM_HAO32MHZ);        //Calibration 31.795MHz
#endif
  SYS_EnableGIE(4,0x3BF);                              //Enable GIE
/*
  DrvPWM0_Open(0,1,2);                           //PWM0 Enable Port 2.0 =PWMO0, Port 2.1 =PWMO1(Set PWM0=PT2.0)
  DrvPWM1_Open(1,1,2);                           //PWM1 Enable Port 2.0 =PWMO0, Port 2.1 =PWMO1(Set PWM1=PT2.1)
  DrvPWM_CountCondition(0X7FFF,0X3FFF);          //PWM0 Duty 0x7FFF, PWM0=PT2.0
                                                 //PWM1 Duty 0x3FFF, PWM1=PT2.1
  DrvGPIO_Open(E_PT2,0x01|0x02,E_IO_OUTPUT);     //PT2.0, PT2.1 Set Output
  DrvTMBC_Clk_Source(1,0);
  DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,0XFFFF);  //TMB Overflow 0xFFFF
*/
#if defined(PWM01)
  DrvPWM0_Open(0,1,2);                           //PWM0 Enable Port 2.0 =PWMO0, Port 2.1 =PWMO1(Set PWM0=PT2.0)
  DrvPWM1_Open(1,1,2);                           //PWM1 Enable Port 2.0 =PWMO0, Port 2.1 =PWMO1(Set PWM1=PT2.1)
  DrvPWM_CountCondition(20,20);          //PWM0 Duty 0x7FFF, PWM0=PT2.0
                                                 //PWM1 Duty 0x3FFF, PWM1=PT2.1
  DrvGPIO_Open(E_PT2,0x01|0x02,E_IO_OUTPUT);     //PT2.0, PT2.1 Set Output
  DrvTMBC_Clk_Source(1,0);
  DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,39);  //TMB Overflow 0xFFFF
#elif defined(PWM23)
  DrvPWM2_Open(0,1,6);                           //PWM0 Enable Port 9.2 =PWMO0, Port 9.3 =PWMO1(Set PWM0=PT9.2)
  DrvPWM3_Open(1,1,6);                           //PWM1 Enable Port 9.2 =PWMO0, Port 9.3 =PWMO1(Set PWM1=PT9.3)
  DrvTMB2PWM_CountCondition(0x7fff,0x3fff);          //PWM0 Duty 0x7FFF, PWM0=PT2.0
                                                 //PWM1 Duty 0x3FFF, PWM1=PT2.1

  //DrvGPIO_PT9_EnableOUTPUT(0x04);
  //DrvGPIO_PT9_EnableOUTPUT(0x08);
  //DrvGPIO_LCDIOOpen(E_PT9,0x04|0x08,E_IO_OUTPUT);
  DrvTMB2_Clk_Source(1,0);
  DrvTMB2_Open(E_TMB_MODE0,E_TMB_NORMAL,0xffff);  //TMB Overflow 0xFFFF
#endif

  InitalRTC();
  MCUSTATUSbits.b_RTCdone=0;

  while(1)
  {
    if(MCUSTATUSbits.b_RTCdone==1)
    {
      SYS_DisableGIE();                              //Disable GIE
      DrvPWM_Out1();
    }
    Delay(1);
  }




  while(1)
  {

	  DrvPWM_Out1();
	  DrvPWM_Out2();
	  DrvPWM_Out3();
	  DrvPWM_Out4();
  }



}

void DrvPWM_Out1(void)
{
    DrvPWM2_Open(0,0,3);//PT2.6
    DrvPWM3_Open(1,1,3);//PT2.7
    DrvTMB2PWM_CountCondition(0x7fff,0x7fff);          //PWM0 Duty 0x7FFF, PWM0=PT2.0
    DrvGPIO_Open(E_PT2,0x40|0x80,E_IO_OUTPUT);     //PT2.6, PT2.7 Set Output
    DrvTMB2_Clk_Source(E_TM_HSCK,0);
    DrvTMB2_Open(E_TMB_MODE0,E_TMB_NORMAL,0xffff);

}
void DrvPWM_Out2(void)
{
    DrvPWM2_Close();
    DrvPWM3_Close();
    DrvTMB2_Close();
    DrvTMB2_ClearTMB();

    DrvPWM2_Open(0,1,0);//PT9.2
    DrvPWM3_Open(1,0,0);//PT9.3
    DrvTMB2PWM_CountCondition(TMB2DOD_PWM3,TMB2DOD_PWM2);


    DrvTMB2_Clk_Source(E_TM_HSCK,0);
    DrvTMB2_Open(E_TMB_MODE0,E_TMB_NORMAL,TMB2COD);
    //DrvTIMER_ClearIntFlag(E_TMB2);
    //DrvTIMER_EnableInt(E_TMB2);
}

void DrvPWM_Out3(void)
{
    DrvPWM0_Close();
    DrvPWM0_Close();
    DrvTMB_Close();
    DrvTMB_ClearTMB();

    DrvPWM0_Open(0,1,1);//PT9.2
    DrvPWM1_Open(1,0,1);//PT9.3
    DrvPWM_CountCondition(TMB2DOD_PWM3,TMB2DOD_PWM2);


    DrvTMBC_Clk_Source(E_TM_HSCK,0);
    DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,TMB2COD);
    //DrvTIMER_ClearIntFlag(E_TMB2);
    //DrvTIMER_EnableInt(E_TMB2);
}

void DrvPWM_Out4(void)
{
    DrvPWM0_Close();
    DrvPWM0_Close();
    DrvTMB_Close();
    DrvTMB_ClearTMB();

    DrvPWM0_Open(0,1,6);//PT9.2
    DrvPWM1_Open(1,0,6);//PT9.3
    DrvPWM_CountCondition(TMB2DOD_PWM3,TMB2DOD_PWM2);


    DrvTMBC_Clk_Source(E_TM_HSCK,0);
    DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,TMB2COD);
    //DrvTIMER_ClearIntFlag(E_TMB2);
    //DrvTIMER_EnableInt(E_TMB2);
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{
  if(DrvRTC_ReadState()&0x2)       //check PTF flag
  {
    DrvRTC_ClearState(E_DRVRTC_CLEAR_ALL);
    DrvRTC_ClearIntFlag();
    MCUSTATUSbits.b_RTCdone=1;
  }
}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}
/*----------------------------------------------------------------------------*/
/* Function Name: InitalRTC()                                                 */
/* Description  : RTC Initialization Subroutines.                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void InitalRTC()
{
  S_DRVRTC_TIME_DATA_T sCurTime;        //Setting Start
  DrvCLOCK_EnableLowOSC(E_EXTERNAL,130000);
  clk_08=0x80800000;  //0x40308 RTCKS=10b, LSXT

  //DRVRTC_CURRENT_TIME or Alarm Time
  DrvRTC_WriteEnable();
  DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
  sCurTime.u8cClockDisplay=0;           //DRVRTC_CLOCK_12//DRVRTC_CLOCK_24
  sCurTime.u8cAmPm=0;                   //DRVRTC_AM//DRVRTC_PM
  sCurTime.u32cSecond=40;
  sCurTime.u32cMinute=59;
  sCurTime.u32cHour=23;
  sCurTime.u32cDay=18;
  sCurTime.u32cMonth=2;
  sCurTime.u32Year=2018;
  sCurTime.u32cDayOfWeek=ComputeWeek(sCurTime.u32Year,sCurTime.u32cMonth,sCurTime.u32cDay);
  sCurTime.u8IsEnableWakeUp=0;          //WK
  DrvRTC_Write(DRVRTC_CURRENT_TIME,&sCurTime);
  DrvRTC_HourFormat(E_DRVRTC_HOUR_24);  //1:12hour 0:24hour
  DrvRTC_Enable();
  DrvRTC_PeriodicTimeEnable(E_DRVRTC_1_8_SEC);
  DrvRTC_ClearIntFlag();
  DrvRTC_EnableInt();                   //Enable RTC interrupt
}

/*----------------------------------------------------------------------------*/
/* Compute Week Subroutines                                                   */
/*----------------------------------------------------------------------------*/
int ComputeWeek(int TempYear, int TempMonth, int TempDay)
{
  int TempWeek;
  if (TempMonth >= 3)
  {
    TempMonth = TempMonth - 2;
  }
  else
  {
    TempMonth = TempMonth + 10;
    TempYear--;
  }

  TempWeek = TempYear + (int)(TempYear / 4) - (int)(TempYear / 100) +(int)(TempYear / 400) + (int)(2.6 * TempMonth - 0.2) + TempDay;
  TempWeek = TempWeek - 7*(int)(TempWeek / 7);
  return (TempWeek);
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
